#  Copyright 2011 unity-launcher-editor-dev.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published
#  by the Free Software Foundation; version 3 only.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#####################################################################

import gtk
from gobject import TYPE_STRING
from utils import FileChooser
from os.path import exists, expanduser, isfile, join


class GroupsDialog(gtk.Dialog):
    """XXX: add docs."""

    def __init__(self, title='Add Group...'):
        super(GroupsDialog, self).__init__(title,
            None, (gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT |
                gtk.DIALOG_NO_SEPARATOR),
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        glade_file = "application.glade"
        self.builder = gtk.Builder()

        glade_path = join("data", glade_file)
        if not exists(glade_path):
            raise Exception(
                "Critical: couldn't find glade file: %s" % glade_path)
        self.builder.add_from_file(glade_path)
        self.add_group_panel = self.builder.get_object('frame4')

        self.ok_button = self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.ok_button.grab_default()
        self.ok_button.set_sensitive(False)
        self.set_alternative_button_order(
            [gtk.RESPONSE_CANCEL, gtk.RESPONSE_OK])

        self.txtGroup =self.builder.get_object('txtGGroup')
        self.txtName = self.builder.get_object('txtGName')
        self.txtName.set_name('Name')
        self.txtCommand = self.builder.get_object('txtGCommand')
        self.txtCommand.set_name('Exec')
        self.cmbTargetEnvironment = self.builder.get_object('cmbTarget')
        self.cmbTargetEnvironment.set_name('TargetEnvironment')

        self.combo_model = gtk.ListStore(TYPE_STRING)
        for target_key in ['Unity', 'Message Menu', 'Unity;Message Menu']:
            self.combo_model.append([target_key])

        celltitle = gtk.CellRendererText()

        self.cmbTargetEnvironment.pack_start(celltitle, False)
        self.cmbTargetEnvironment.add_attribute(celltitle, 'text', 0)
        self.cmbTargetEnvironment.set_model(self.combo_model)
        self.cmbTargetEnvironment.set_active(0)

        self.txtName.connect('changed', self.on_change)
        self.txtCommand.connect('changed', self.on_change)
        self.builder.get_object('cmdGRun').connect(
            'clicked', self.open_executable)

        self.vbox.pack_start(self.add_group_panel, True, True, 0)

    def populate(self, values):
        try:
            self.txtGroup.set_text(values['GroupName'])
            self.txtGroup.set_sensitive(False)
            self.txtName.set_text(values['Name'])
            self.txtCommand.set_text(values['Exec'])
        except:
            pass
        try:
            for n in range(len(self.combo_model)):
                if self.combo_model[n][0] == values['TargetEnvironment']:
                    self.cmbTargetEnvironment.set_active(n)
        except:
            pass
        
    def run(self, destroy=True):
        """Returns True if yes was clicked, False otherwise."""
        resp = super(GroupsDialog, self).run()
        obj = None
        if resp == gtk.RESPONSE_OK:
            obj = {'GroupName': self.txtGroup.get_text(),
                   'Name': self.txtName.get_text(),
                   'Exec': self.txtCommand.get_text(),
                   'TargetEnvironment': (
                      self.cmbTargetEnvironment.get_active_text()),
            }
        if destroy:
            self.destroy()
        if resp == gtk.RESPONSE_OK:
            return True, obj
        else:
            return False, None

    def on_change(self, widget):
        if (self.txtName.get_text_length() == 0 or 
            self.txtCommand.get_text_length() == 0):
            self.ok_button.set_sensitive(False)
        else:
            self.ok_button.set_sensitive(True)

    def open_executable(self, widget):
        dialog = FileChooser()
        response, filename = dialog.run()
        if response:
            self.txtCommand.set_text(filename)


if __name__ == "__main__":
    dialog = GroupsDialog()
    result, group = dialog.run()
