# -*- coding: utf-8 -*-
#####################################################################
#  Laudeci Oliveira <laudeci@ubuntu.com>
#  Ursula Junques <ursinha@ubuntu.com>
#
#  Copyright 2011 unity-launcher-editor-dev.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published
#  by the Free Software Foundation; version 3 only.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#####################################################################

import gtk
from gtk.gdk import Pixbuf
import gobject
from desktop import DesktopParser
from os.path import basename, exists, expanduser, join
from utils import DialogType, FileChooser, get_icon, MessageBox, nullstring
from groupsdialog import GroupsDialog

# Constants for column mapping
(COLUMN_ICON,
 COLUMN_NAME,
 COLUMN_DESKTOP,
 COLUMN_PATH) = range(4)

#constantes for toolbar buttons
(TOOLBUTTON_ADD,
TOOLBUTTON_EDIT,
TOOLBUTTON_REMOVE) = range(3)

class MainWindow(object):

    builder = gtk.Builder()

    def __init__(self, controller = None, glade_file="application.glade"):
        self.controller = controller
        glade_path = join("data", glade_file)
        if not exists(glade_path):
            raise "Critical: couldn't find glade file: %s" % glade_path

        window_name = "window1"
        self.builder.add_from_file(glade_path)
        self.window = self.builder.get_object(window_name)

        # launcher Objects
        self.listview_launcher = self.builder.get_object('ListViewLauncher')
        self.LauncherModel = gtk.ListStore(Pixbuf, gobject.TYPE_STRING,
            gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
        render_pixbuf = gtk.CellRendererPixbuf()
        render_text = gtk.CellRendererText()
        col = gtk.TreeViewColumn()
        col.pack_start(render_pixbuf, expand=False)
        col.add_attribute(render_pixbuf, 'pixbuf', COLUMN_ICON)
        col.pack_start(render_text, expand=True)
        col.add_attribute(render_text, 'text', COLUMN_NAME)
        self.listview_launcher.append_column(col)
        self.listview_launcher.set_model(self.LauncherModel)

        # groups objects
        self.tvwGroups = self.builder.get_object('tvwGroups')
        self.GroupsModel = gtk.TreeStore(gobject.TYPE_STRING,
            gobject.TYPE_STRING)
        col = gtk.TreeViewColumn('Quicklist Groups',
            gtk.CellRendererText(), text=0)
        self.tvwGroups.append_column(col)

        cell_renderer = gtk.CellRendererText();
        #cell_renderer.set_property('editable', True);
        col = gtk.TreeViewColumn('', cell_renderer, text=1);
        self.tvwGroups.append_column(col)

        self.tvwGroups.set_model(self.GroupsModel)

        # info Widgets
        self.imgIcon = self.builder.get_object('imgIcon')
        self.txtName = self.builder.get_object('txtName')
        self.txtName.set_name('Name')
        self.txtComment = self.builder.get_object('txtComment')
        self.txtComment.set_name('Comment')
        self.txtCommand = self.builder.get_object('txtCommand')
        self.txtCommand.set_name('Exec')
        
        self.chkTerminal = self.builder.get_object('chkTerminal')
        self.chkTerminal.set_name('Terminal')

        # button widgets
        self.cmdIcon = self.builder.get_object('cmdIcon')
        self.cmdSave = self.builder.get_object('cmdSave')
        
        #toolbar widgets
        self.btnRemove = self.builder.get_object('btnRemove')
        self.btnEdit = self.builder.get_object('btnEdit')

        # set signals
        self.setup_signals()

    def setup_signals(self):
        # window close
        self.window.connect('delete-event', gtk.main_quit)
        
        #buttons event
        self.builder.get_object('cmdClose').connect('clicked', gtk.main_quit)
        self.builder.get_object('cmdAdd').connect('clicked', self.add_new_launcher)
        self.cmdSave.connect('clicked', self.save_launchers)
        self.builder.get_object('cmdRemove').connect('clicked', self.remove_from_launcher)
        self.cmdIcon.connect('clicked', self.load_icon_from_file)
        self.builder.get_object('cmdRun').connect('clicked', self.open_executable)
        self.chkTerminal.connect('toggled', self.chk_toggle)
        
        #editable events
        self.txtName.connect('focus-out-event', self.on_txt_focus_out)
        self.txtComment.connect('focus-out-event', self.on_txt_focus_out)
        self.txtCommand.connect('focus-out-event', self.on_txt_focus_out)
        
        #list group signals
        self.tvwGroups.connect('cursor-changed', self.tvwgroups_row_change)
        
        #group signals
        self.builder.get_object('btnAdd').connect('clicked', self.add_remove_group, TOOLBUTTON_ADD)
        self.btnRemove.connect('clicked', self.add_remove_group, TOOLBUTTON_REMOVE)
        self.btnEdit.connect('clicked', self.add_remove_group, TOOLBUTTON_EDIT)
        
    def show(self):
        self.window.show_all()
    
    def normalize_path(self, str_path):
        file_name= basename(str_path)
        str_path='%s/.local/share/applications/%s' % (expanduser('~'),file_name)
        
        #if not '/home' in str_path:
        #    str_path='%s.local/share/applications/%s' % (expanduser('~'),str_path)
        #    #str_path='%s/Desktop/applications/%s' % (expanduser('~'),str_path)
        #else:
        #    file_name= basename(str_path)
        #    str_path='%s/Desktop/applications/%s' % (expanduser('~'),file_name)
        return str_path    
    
    def save_launchers(self, widget):
        dialog = MessageBox( self.window,
                                window_title='Save' ,
                                action_text='Save changes',
                                message='Do you want to save your changes?',
                                dlg_type=DialogType.DIALOG_CONFIRMATION,
                                label_text=None
                                )
        result, text = dialog.run()                   
        if result:   
            for n in self.LauncherModel:
                obj=n[COLUMN_DESKTOP]
                fli = open(self.normalize_path( n[COLUMN_PATH]),'w')
                
                obj.write(fli)
                fli.close()
                launchers=[]
                for launcher in self.LauncherModel:
                    launchers.append(self.normalize_path(launcher[COLUMN_PATH]))
                if self.controller:
                    self.controller.save_unity_list(launchers)
    def chk_toggle(self, widget):
        # only run when gaving the focus
        # this will prevent execution when setting the value by code.
        
        if widget.is_focus():
            value = widget.get_active()
            model, tree_iter = self.get_selected_launcher_row()
            objDesktop = model[tree_iter][COLUMN_DESKTOP]
            option = widget.get_name()
            objDesktop.set(option, value)
            
    def on_txt_focus_out(self, widget, event):
        ilength = 0
        if  isinstance(widget, gtk.TextView):
            buffer = widget.get_buffer()
            value = buffer.get_text(buffer.get_start_iter() , buffer.get_end_iter())
            ilength = len(value)
        else:
            ilength = widget.get_text_length() 
            value = widget.get_text()
            
        if ilength > 0:
            model, tree_iter = self.get_selected_launcher_row()
            objDesktop = model[tree_iter][COLUMN_DESKTOP]
            option = widget.get_name()
            
            if option == 'Name':
                model[tree_iter][COLUMN_NAME] = value
                
            objDesktop.set(option, value)
    
    def add_remove_group(self, widget, operation):
        model, tree_iter = self.get_selected_launcher_row()
        if not model:
            return
        
        objDesktop = model[tree_iter][COLUMN_DESKTOP]
        
        if operation == TOOLBUTTON_ADD:
            #TODO : update ayatana entry
            
            dialog = GroupsDialog()
            result, group = dialog.run()
             
            if result:            
                gModel = self.tvwGroups.get_model()
                root = gModel.append(None, (group['GroupName'], ''))
                
                shortcut_group_name = "%s Shortcut Group" % group['GroupName']
                
                gModel.append(root, ('Name', group['Name']))
                gModel.append(root, ('Exec', group['Exec']))
                gModel.append(root, ('TargetEnvironment', group['TargetEnvironment']))
                
                items = objDesktop.get('X-Ayatana-Desktop-Shortcuts')
                if items is None:
                    items = []
                else:
                    items =[n for n in items if n != '']
                items.append(group['GroupName'])
                objDesktop.set('X-Ayatana-Desktop-Shortcuts', items)
                
                objDesktop.append_section(shortcut_group_name)
                objDesktop.set('Name', group['Name'] , None, shortcut_group_name)           
                objDesktop.set('Exec', group['Exec'] , None, shortcut_group_name)
                objDesktop.set('TargetEnvironment', group['TargetEnvironment'] , None, shortcut_group_name)
                
                
                
        elif operation == TOOLBUTTON_REMOVE:
            dialog = MessageBox(self.window,
                                window_title='Remove' ,
                                action_text='Remove Group',
                                message='You are about to remove a Quick List Group, are you sure?',
                                dlg_type=DialogType.DIALOG_CONFIRMATION,
                                label_text=None
                                )
            result = dialog.run()
                 
            if result[0]:
                selection = self.tvwGroups.get_selection()
                gModel, titer = selection.get_selected()
                root_iter = gModel.iter_parent(titer)
                
                if not root_iter:
                    root_iter = titer
                
                shortcut_group_name = "%s Shortcut Group" % gModel.get_value(titer, 0)
                gModel.remove(root_iter)
                items = objDesktop.get('X-Ayatana-Desktop-Shortcuts')
                items.remove(shortcut_group_name)
                objDesktop.remove_section(shortcut_group_name)
                objDesktop.set('X-Ayatana-Desktop-Shortcuts',items)
                
        elif operation == TOOLBUTTON_EDIT:               
            selection = self.tvwGroups.get_selection()
            gModel, titer = selection.get_selected()
            
            root_iter = gModel.iter_parent(titer)
            
            if not root_iter:
                root_iter = titer
            count = gModel.iter_n_children(root_iter)
            
            shortcut_group_name = "%s Shortcut Group" % gModel.get_value(root_iter, 0)
            
            items = {'GroupName': gModel.get_value(root_iter, 0)}
            
            for n in range(count):
                it = gModel.iter_nth_child(root_iter, n)
                items[ gModel.get_value(it, 0)]= gModel.get_value(it, 1)
            dialog = GroupsDialog()
            dialog.populate(items)
            result, group = dialog.run()
            
            if result:
                #gModel.set(root_iter,0,group['Name'])
                it = gModel.iter_nth_child(root_iter, 0)
                gModel.set(it,1,group['Name'])
                it = gModel.iter_nth_child(root_iter, 1)
                gModel.set(it,1,group['Exec'])
                it = gModel.iter_nth_child(root_iter, 2)
                gModel.set(it,1,group['TargetEnvironment'])
                
                objDesktop.set('Name', group['Name'] , None, shortcut_group_name)           
                objDesktop.set('Exec', group['Exec'] , None, shortcut_group_name)
                objDesktop.set('TargetEnvironment', group['TargetEnvironment'] , None,shortcut_group_name)
            
            
            
    def remove_from_launcher(self, widget):
        """Signal handler for button to remoce an item from the Launcher List"""
        
        model, tree_iter = self.get_selected_launcher_row()
        if tree_iter:
            dialog = MessageBox(self.window,
                                window_title='Remove' ,
                                action_text='Remove Launcher',
                                message='You are about to remove a Launcher item, are you sure?',
                                dlg_type=DialogType.DIALOG_CONFIRMATION,
                                label_text=None
                                )
            result, text = dialog.run()                   
            if result:
                #get the current index on the list
                indx = model.get_path(tree_iter)[0]
                model.remove(tree_iter)
                # try to select the previous item in the list
                #or the next if the list still having data
                icount = len(model)
                print indx, icount
                
                if indx == icount and icount > 0:
                    print 'indx-1'
                    self.listview_launcher.set_cursor((indx - 1,))
                elif indx < icount and icount > 0:
                    print 'indx'
                    self.listview_launcher.set_cursor((indx,))
                else:
                    print 'indx+1'
                    self.listview_launcher.set_cursor((indx + 1,))
                        
        else:
            MessageBox(self.window,window_title='Remove' ,
                       action_text='Remove Launcher',
                       message='No launcher selected',
                       dlg_type=DialogType.DIALOG_CONFIRMATION,
                       label_text=None
                       ).run()

    def add_new_launcher(self, widget):
        """Signal handler for button to add a new item into the Launcher List"""
        
        dialog = MessageBox(self.window,
                                window_title='Add Launcher' ,
                                action_text='Add a new launcher...' ,
                                message='Type a launcher name to create a new launcher.',
                                dlg_type=DialogType.DIALOG_INPUT,
                                label_text='Name:'
                                )

        result, text = dialog.run()
        print result, text 
        if result and len(text) > 0:
            desktop_parser = DesktopParser()
            desktop_parser.set('Name', text)
            desktop_parser.set('Terminal', False)
            path = join(expanduser('~/.local/share/applications/'),
                '%s.desktop' % text)
            tree_iter = self.LauncherModel.append((get_icon('wft'), text, desktop_parser,
                path))
            #selects the inserted row
            self.listview_launcher.set_cursor(self.LauncherModel.get_path(tree_iter))

    def open_executable(self, widget):
        dialog = FileChooser()
        response, filename = dialog.run()
        if response:
            self.txtCommand.set_text(filename)
            model, tree_iter = self.get_selected_launcher_row()
            objDesktop = model[tree_iter][COLUMN_DESKTOP]
            objDesktop.set(self.txtCommand.get_name(), filename)
            
    def load_icon_from_file(self, widget):
        
        # XXX: Here maybe we could make a list with all mimetypes and patterns
        # somewhere else and just iterate over them here.
        filter = gtk.FileFilter()
        filter.set_name("Images")
        filter.add_mime_type("image/png")
        filter.add_mime_type("image/jpeg")
        filter.add_mime_type("image/gif")
        filter.add_mime_type("image/svg+xml")
        filter.add_pattern("*.png")
        filter.add_pattern("*.jpg")
        filter.add_pattern("*.gif")
        filter.add_pattern("*.tif")
        filter.add_pattern("*.xpm")
        filter.add_pattern("*.svg")
        
        dialog = FileChooser(filters=filter)
        response, filename = dialog.run()
        if response:
            pixbuf = get_icon(filename)
            self.imgIcon.set_from_pixbuf(pixbuf)
            # TODO: add code to write changes here
            # the best way is to put an apply button?
            model, tree_iter = self.get_selected_launcher_row()
            objDesktop = model[tree_iter][COLUMN_DESKTOP]
            model[tree_iter][COLUMN_ICON] = pixbuf
            objDesktop.set('Icon', filename)
    
    def get_selected_launcher_row(self):
        selection = self.listview_launcher.get_selection()
        return selection.get_selected()
    
    def listview_launcher_row_change(self, data=None):
        
        #disable groups toobar buttons
        self.btnEdit.set_sensitive(False)
        self.btnRemove.set_sensitive(False)
            
        model, tree_iter = self.get_selected_launcher_row()

        objDesktop = model[tree_iter][COLUMN_DESKTOP]

        # show properties
        self.imgIcon.set_from_pixbuf(model[tree_iter][COLUMN_ICON])

        self.txtName.set_text(nullstring(objDesktop.get('Name')))
        self.txtComment.get_buffer().set_text(
            nullstring(objDesktop.get('Comment')))
        self.txtCommand.set_text(nullstring(objDesktop.get('Exec')))
        self.chkTerminal.set_active(objDesktop.get('Terminal'))

        items = objDesktop.get('X-Ayatana-Desktop-Shortcuts')
        
        gModel = self.tvwGroups.get_model()
        gModel.clear()

        if items:
            if isinstance(items,str):
                items=[items]
                
            for group in items:
                if group != '':
                    root = gModel.append(None, (group, ''))
                    for key in ('Name', 'Exec', 'TargetEnvironment'):
                        opt = objDesktop.get(key, "%s Shortcut Group" % group)
                        if opt:
                            gModel.append(root, (key, opt))
                        
    def tvwgroups_row_change(self, data=None):
        selection = self.tvwGroups.get_selection()
        gModel, titer = selection.get_selected()
        
        if not gModel or not titer:
            return
        
        root_iter = gModel.iter_parent(titer)
        
        if not root_iter:
            root_iter = titer
        if root_iter:
            self.btnEdit.set_sensitive(True)
            self.btnRemove.set_sensitive(True)
        else:
            self.btnEdit.set_sensitive(True)
            self.btnRemove.set_sensitive(True)
            
